import java.util.*;
public class Block extends PhysicsElement  {
   private static int id=0;
   private final int fix;
   private final float mass;
   private final float gravityAccel = 9.80665f;
   private boolean damper = false;
   private final float dampingFactor = 0.1f;
   protected Vector2D pos_t;
   private Vector2D pos_tPlusDelta;
   private Vector2D speed_t;
   private Vector2D speed_tPlusDelta;
   private Vector2D gravity;
   private Vector<PhysicsElement> elements;  // This Vector is a growable array of objects
                            // it has nothing to do with Vector2D.
   private Block(){ // nobody can create a block without state
      super(id++);
      mass=0;
      fix = 1;
   }  
   public Block(float mass, Vector2D position, Vector2D speed, int fix){
      super(id++);
      this.mass = mass;
      pos_t = position;
      this.speed_t = speed;
      elements = new Vector<PhysicsElement>();
      this.fix = fix;        // This parameter fix the block to its position, if fix = 0.
   }
   public void attachSpring (Spring spring) {
     elements.add(spring);
   }
   public void detachSpring (Spring spring) {
     elements.remove(spring);
   }
   public void attachString (MyString string) {
     elements.add(string);
   }
   public void detachString (MyString string) {
     elements.remove(string);
   }
   public Vector2D getPosition() {
      return pos_t;
   }
   public void setGravity(boolean grav){ // Sets on or off the gravity force to the block.
      if(grav){                          // Gravity acceleration is fixed to 9.80665 [m/s^2].
         gravity = new Vector2D(0,gravityAccel);
         gravity = gravity.times(mass);
         damper = true;                  // Activates damping factor when gravity activates,
      }                                  // in order to give stability to the system.
      else{
         gravity = new Vector2D(0,0);
         damper = false;
      }
   }
   public void computeNextState(double delta_t) {
     Vector2D force = new Vector2D();
     Spring spring;
     MyString string;
     if(fix != 0){                 // If block is fixed, the total force applied to the block is zero.
        for (int i=0; i<elements.size(); i++) {
            if(elements.get(i) instanceof Spring){
                spring = (Spring) elements.get(i);
                if (spring != null) force = force.plus(spring.getForce(this));
            }
            if(elements.get(i) instanceof MyString){
                string = (MyString) elements.get(i);
                if (string != null) force = force.plus(string.getForce(this));
            }
        }
     force = force.plus(gravity);        // Add gravity force and friction to the total force.
     if(damper)
         force = force.plus(speed_t.times((-1)*dampingFactor));
     }
     Vector2D a = force.times(1/mass);
     pos_tPlusDelta = pos_t.plus(speed_t.times(delta_t)).plus(a.times(delta_t*delta_t/2));
     speed_tPlusDelta = speed_t.plus(a.times(delta_t));
   }
   public void updateState(){
     pos_t = pos_tPlusDelta;
     speed_t = speed_tPlusDelta;
   }
   public String getDescription() {
      return "Block#" + getId() + ": "+getPosition().getDescription()+",";
   }
   public String getState() {
         return getPosition()+",";
   }
} 
    
    
           
      